<?php

namespace MGModule\EasyDCIM\Helpers;

use Exception;
use MGModule\EasyDCIM\MgLibs\Database\Models\Currencies;
use MGModule\EasyDCIM\MgLibs\Database\Models\HostingConfigOptions;
use MGModule\EasyDCIM\MgLibs\Database\Models\Pricing;
use MGModule\EasyDCIM\MgLibs\Database\Models\Product;
use MGModule\EasyDCIM\MgLibs\Database\Models\ProductConfigGroups;
use MGModule\EasyDCIM\MgLibs\Database\Models\ProductConfigLinks;
use MGModule\EasyDCIM\MgLibs\Database\Models\ProductConfigOptions;
use MGModule\EasyDCIM\MgLibs\Database\Models\ProductConfigOptionsSub;
use MGModule\EasyDCIM\MgLibs\Lang;

/**
 * Description of ConfigurableOptions
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class ConfigurableOptions
{
    /*
     * Available Configurable option typ
     */

    private static $availableType = [
        1 => 'Dropdown',
        2 => 'Radio',
        3 => 'Yes/No',
        4 => 'Quantity',
    ];

    public static function getAll($serviceID)
    {
        $fields = [];
        foreach (self::getFieldsWithValues($serviceID) as $value)
        {
            $fieldName          = explode("|", $value->options->optionname)[0];
            $fieldValue         = explode("|", $value->values->optionname)[0];
            $fields[$fieldName] = $fieldValue;
        }
        return $fields;
    }

    private static function getFieldsWithValues($serviceID)
    {
        return HostingConfigOptions::where('relid', $serviceID)->with('options', 'values')->get();
    }

    /*
     * Create Configurable Options Group
     * 
     * $params string  $productName, $moduleName
     * @return object $optionsGroup insteandof ProductConfigGroups
     */

    public static function prepareConfigurableOptions($productID)
    {
        $product      = self::getProduct($productID);
        self::checkExcistsOptionsGroup($product->id);
        $optionsGroup = self::createOptionsGroup($product);
        self::assignGroupToProduct($optionsGroup, $product);
        return $optionsGroup;
    }

    /*
     * Configurable options Adpter
     * 
     * $params mixed
     * @return mixed
     */

    public static function typeAdapter($type)
    {
        $table = self::$availableType;
        if (!is_numeric($type))
        {
            $table = array_flip($table);
        }
        return $table[$type];
    }

    /*
     * Get iroduct inforamtion
     * 
     * $params integer $productID
     * @return object insteandof Product
     */

    private static function getProduct($productID)
    {
        return Product::where('id', $productID)->first();
    }

    /*
     * Check if configurable options exists
     * 
     * $params  integer $productId
     * @throw error 
     */

    public static function checkExcistsOptionsGroup($productID)
    {
        $check = ProductConfigLinks::where('pid', $productID)->first();
        if (!is_null($check))
        {
            throw new Exception(Lang::T('configurableOptionExsits'));
        }
    }

    /*
     * Create configurable options group
     * 
     * $params  object $product insteandof Product
     * @throw database error
     */

    private static function createOptionsGroup(Product $product)
    {
        $optionsGroup              = new ProductConfigGroups();
        $optionsGroup->name        = "Configurable options for " . $product->name . " product";
        $optionsGroup->description = "Auto generated by module " . $product->servertype;
        $optionsGroup->save();
        return $optionsGroup;
    }

    /*
     * Assign Configurable Options Group to Product
     * 
     * $params object $optionsGroup insteandof ProductConfigGroups
     * @throw database error
     */

    private static function assignGroupToProduct(ProductConfigGroups $optionsGroup, Product $product)
    {

        if (is_null($optionsGroup))
        {
            throw new Exception(Lang::T('groupNotExsists'));
        }
        $assignProduct      = new ProductConfigLinks();
        $assignProduct->gid = $optionsGroup->id;
        $assignProduct->pid = $product->id;
        $assignProduct->save();
    }

    /*
     * Create configurable options
     * 
     * $params  string $name, $type integer $min, $max, $order, $hidden
     * @return $options insteand of ProductConfigOptions
     * @throw database error
     */

    public static function createOptions(ProductConfigGroups $optionsGroup, $name, $type, $min = 0, $max = 0, $order = 0, $hidden = 0)
    {
        $options             = new ProductConfigOptions();
        $options->gid        = $optionsGroup->id;
        $options->optionname = $name;
        $options->optiontype = $type;
        $options->qtyminimum = $min;
        $options->qtymaximum = $max;
        $options->order      = $order;
        $options->hidden     = $hidden;
        $options->save();

        return $options;
    }

    /*
     * Create sub-options to configurable options
     * Function automatically create pricing for configurable options
     * 
     * $params  string $configID integer $optionName, $sort, $hidden
     * @return $options insteand of ProductConfigOptions
     * @throw database error
     */

    public static function createSubOptions($configID, $optionName, $sort = 0, $hidden = 0)
    {
        $subOptions             = new ProductConfigOptionsSub();
        $subOptions->configid   = $configID;
        $subOptions->optionname = $optionName;
        $subOptions->sortorder  = $sort;
        $subOptions->hidden     = $hidden;
        $subOptions->save();
        self::createPricingForOptions($subOptions->id);
    }

    /*
     * Get all currencies
     * 
     * @return array $currencies
     * @throw database error
     */

    private static function getCurrencies()
    {
        $currencies    = Currencies::get();
        $allCurrencies = [];
        foreach ($currencies as $currency)
        {
            $allCurrencies[] = $currency->id;
        }
        return $allCurrencies;
    }

    /*
     * Get all currencies and create pricing for each one
     * 
     * @params integer $subOptionID
     */

    private static function createPricingForOptions($subOptionID)
    {
        foreach (self::getCurrencies() as $currency)
        {
            self::addPricing($subOptionID, $currency);
        }
    }

    /*
     * Add pricing for sub-options
     * 
     * @params integer $subOptionID, $currencyID string $type
     * @throw database error
     */

    private static function addPricing($subOptionID, $currencyID, $type = 'configoptions')
    {
        $pricing               = new Pricing();
        $pricing->type         = $type;
        $pricing->currency     = $currencyID;
        $pricing->relid        = $subOptionID;
        $pricing->msetupfee    = 0;
        $pricing->qsetupfee    = 0;
        $pricing->ssetupfee    = 0;
        $pricing->asetupfee    = 0;
        $pricing->bsetupfee    = 0;
        $pricing->tsetupfee    = 0;
        $pricing->monthly      = 0;
        $pricing->quarterly    = 0;
        $pricing->semiannually = 0;
        $pricing->annually     = 0;
        $pricing->biennially   = 0;
        $pricing->triennially  = 0;
        $pricing->save();
    }

}
