<?php

namespace MGModule\EasyDCIM\MgLibs\Process;

use Exception;
use MGModule\EasyDCIM\MgLibs\Database\Models\Configuration;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Exceptions\FatalError;
use MGModule\EasyDCIM\MgLibs\Exceptions\System;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Process\AbstractMainDriver;
use MGModule\EasyDCIM\MgLibs\Process\MainInstance;
use MGModule\EasyDCIM\MgLibs\Smarty;

/**
 * Main Abstract Controller 
 * 
 * @author Michal Czech <michael@modulesgarden.com>
 * @SuppressWarnings(PHPMD)
 */
abstract class AbstractMainDriver
{

    /**
     * Singleton Instance
     * 
     * @var mainController
     */
    static private $_instance;

    /**
     * This Var define Debug Mode in Module
     * 
     * @var boolean
     */
    public $_debug = false;

    /**
     * Mark 
     * 
     * $var boolean
     */
    public $isLoaded;

    /**
     * Load Configuration
     * 
     * @var Configuration
     */
    private $_configuration;

    /**
     * Is Loaded From Admin Area
     * 
     * @var boolean 
     */
    private $_isAdmin = false;

    /**
     * Key For Data Encryption
     * 
     * @var string 
     */
    private $encryptSecureKey;

    /**
     * Main Namespace
     * 
     * @var string
     */
    private $_mainNamespace;
    private $_mainDIR;

    /**
     * Disable Contruct && Clone
     */
    private final function __construct()
    {
        ;
    }

    private final function __clone()
    {
        ;
    }

    /**
     * Get SingleTon Instance
     * 
     * @return AbstractMainDriver
     */
    public static function I($force = false, $configs = [])
    {
        if (empty(self::$_instance) || $force)
        {
            $class = get_called_class();

            MainInstance::setInstanceName($class);

            self::$_instance                 = new $class();
            self::$_instance->_mainNamespace = substr(__NAMESPACE__, 0, strpos(__NAMESPACE__, '\MgLibs'));
            self::$_instance->_mainDIR       = call_user_func([$class, 'getMainDIR']);

            $class = self::$_instance->_mainNamespace . '\Configuration';

            self::$_instance->_configuration = new $class();

            foreach ($configs as $name => $value)
            {
                self::$_instance->_configuration->{$name} = $value;
            }

            self::$_instance->isLoaded = true;

            if (!empty(self::$_instance->_configuration->debug))
            {
                self::$_instance->_debug = true;
            }

            Lang::getInstance(self::$_instance->_mainDIR . DS . 'Langs');
        }

        return self::$_instance;
    }

    /**
     * 
     * @return main\Configuration
     */
    function configuration()
    {
        return $this->_configuration;
    }

    /**
     * Load Database Migrations 
     *
     * @return  Migrations
     */
    function migrations()
    {
        return new Migrations($this->getMainNamespace());
    }

    public function isAdmin($status = null)
    {
        if ($status !== null)
        {
            $this->_isAdmin = $status;
        }
        return $this->_isAdmin;
    }

    public function isDebug()
    {
        return $this->_debug;
    }

    /**
     * Return Main Namespace
     * 
     * @return string
     */
    function getMainNamespace()
    {
        return $this->_mainNamespace;
    }

    /**
     * Return Enrypt Key
     * 
     * @return string
     */
    public function getEncryptKey()
    {

        if (empty($this->encryptSecureKey))
        {
            $this->encryptSecureKey = hash('sha256', $GLOBALS['cc_encryption_hash'] . $this->configuration()->encryptHash, TRUE);
        }

        return $this->encryptSecureKey;
    }

    function setMainLangContext()
    {
        Lang::setContext(lcfirst($this->getType()) . ($this->isAdmin() ? 'AA' : 'CA'));
    }

    /**
     * Process Controllers
     * 
     * @param string $controller controller name
     * @param array $input input array
     * @param string $type type of request
     * @return array
     * @throws main\MgLibs\exceptions\System
     * @throws main\MgLibs\exceptions\System
     */
    function runControler($controller, $action = 'index', $input = [], $type = 'HTML')
    {
        try
        {
            $className = $this->getMainNamespace() . "\\Controllers\\" . $this->getType() . "\\" . ($this->_isAdmin ? 'Admin' : 'Clientarea') . "\\" . $controller;

            if (!class_exists($className))
            {
                throw new System("Unable to find page");
            }

            $controllerOBJ = new $className($input);
            // display the page or not
            if (method_exists($controllerOBJ, "isActive") && !$controllerOBJ->{"isActive"}())
                throw new System("No access to this page");


            if (!method_exists($controllerOBJ, $action . $type))
            {
                throw new System("Unable to find Action: " . $action . $type);
            }

            Lang::stagCurrentContext('generate' . $controller);

            Lang::addToContext(lcfirst($controller));

            Smarty::I()->setTemplateDir(self::I()->getModuleTemplatesDir() . DS . 'pages' . DS . lcfirst($controller));

            $result = $controllerOBJ->{$action . $type}($input);


            switch ($type)
            {
                case 'HTML':
                    if (empty($result['tpl']))
                    {
                        throw new System("Provide Template Name");
                    }

                    $success    = isset($result['vars']['success']) ? $result['vars']['success'] : false;
                    $error      = isset($result['vars']['error']) ? $result['vars']['error'] : false;
                    $fatalError = isset($result['vars']['fatalError']) ? $result['vars']['fatalError'] : false;
                    $result     = Smarty::I()->view($result['tpl'], $result['vars']);
                    break;
                default:
                    $success    = isset($result['success']) ? $result['success'] : false;
                    $error      = isset($result['error']) ? $result['error'] : false;
            }

            Lang::unstagContext('generate' . $controller);

            return [
                $result,
                $success,
                $error,
                $fatalError
            ];
        }
        catch (Exception $ex)
        {
            if ($ex instanceof FatalError)
            {
                throw new Exception(Lang::absoluteT('serverAA', $ex->getMessage()));
            }
            Lang::unstagContext('generate' . $controller);
            throw $ex;
            return false;
        }
    }

    /**
     * Process Custom Controllers
     * 
     * @param string $controller controller name
     * @param array $input input array
     * @return array
     * @throws main\MgLibs\exceptions\System
     */
    function runCustomControler($controller, $action = 'index', $input = [], $type = 'HTML')
    {


        try
        {
            $className = $this->getMainNamespace() . "\\Controllers\\" . $this->getType() . "\\" . ($this->_isAdmin ? 'Admin' : 'Clientarea') . "\\" . $controller;


            if (!class_exists($className))
            {
                throw new System("Unable to find page");
            }

            $controllerOBJ = new $className($input);
            // display the page or not
            if (method_exists($controllerOBJ, "isActive") && !$controllerOBJ->{"isActive"}())
                throw new System("No access to this page");


            if (!method_exists($controllerOBJ, $action . $type))
            {
                throw new System("Unable to find Action: " . $action . $type);
            }
            return $controllerOBJ->{$action . $type}($input);
        }
        catch (Exception $ex)
        {
            throw $ex;
            return false;
        }
    }

    /**
     * Dump data
     * 
     * @param mixed $input
     */
    static function dump($input)
    {
        if (self::I()->isDebug())
        {
            echo "<pre>";
            print_r($input);
            echo "</pre>";
        }
    }

    abstract public function getAssetsURL();

    abstract public function getType();

    public static function getMainDIR()
    {
        return false;
    }

    public static function getUrl()
    {
        return false;
    }

    public function isActive()
    {
        return true;
    }

}
