<?php

namespace MGModule\EasyDCIM\Repositories\Actions;

use Exception;
use MGModule\EasyDCIM\Helpers\ConfigurableOptions;
use MGModule\EasyDCIM\Helpers\CustomFields;
use MGModule\EasyDCIM\Helpers\FileDir;
use MGModule\EasyDCIM\Helpers\OsTemplate;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Interfaces\IClient;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Orders\OrderCreate;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Orders\OrderCriteria;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Orders\OrderPart;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Orders\OrderPartContainer;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Orders\OrderService;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Users\UserContainer;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Users\UserDetails;
use MGModule\EasyDCIM\MgLibs\FileReader\FileReader;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Helpers\Emails;
use const DS;

/**
 * Description of Create
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class Create extends AbstractRepository
{
    /*
     * Array of disabled custom options
     */

    private $disabledOptions = [
        'model',
        'location',
        'template',
        'bandwidth',
        'additionalIpsNumber',
        'hdd',
        'ssd',
        'ram',
        'cpuCores',
        'ram',
        'hostname',
        'username',
        'password',
        'monthly_traffic_limit',
        'additional_ips'
    ];

    /**
     * Add new order and create user if not exists
     * 
     */
    public function create()
    {
        $this->getCustomInformation();
        $this->checkCusomFieldsIsEmpoty();
        return $this->addOrder(new ClientAdapter($this->params));
    }

    /**
     * Check Order id is empty
     * 
     * @throw error if is set
     * 
     */
    private function checkCusomFieldsIsEmpoty()
    {
        $orderField = CustomFields::get($this->params['params']['packageid'], $this->params['params']['serviceid'], 'OrderID');
        if (!empty($orderField))
        {
            throw new Exception(Lang::T('OrderIDIsNoEmpty'));
        }
    }

    /**
     * Create new order
     * Save custom fields Order ID, server ID 
     * 
     * $param $client insteandof ClientAdapter
     * 
     */
    private function addOrder(IClient $client)
    {

        $clientID = $this->getClientID($client);

        $parts = $this->getParts($client);

        $criteriaModel = new OrderCriteria();
        $criteriaModel->setModel($client->getModelID())
                ->setLocation($client->getLocationID())
                ->setRequirePdu($client->getBaseFeatures()['requirePdu'])
                ->setRequireSwitch($client->getBaseFeatures()['requireSwitch'])
                ->setRequireParts($this->checkPartIsActive($client))
                ->setParts($parts);

        try
        {
            $templateID = (new OsTemplate($this->params))->getOSTemplateID($client->getOsTemplate());
        }
        catch (Exception $ex)
        {
            $templateID = $client->getOsTemplate();
        }

        $serviceModel = new OrderService();
        $serviceModel->setHostname($client->getDomain())
                ->setTemplate($templateID)
                ->setUsername($client->getUsername())
                ->setPassword($client->getPassword())
                ->setMonthlyTrafficLimit($client->getBandwidth())
                ->setAccessLevel($client->getAcccessLevel())
                ->setCustomFields($this->getCustomInformation());

        if(!empty($client->getAdditionalIPAddress()) || $client->getAdditionalIPAddress() != 0){
            $serviceModel->setAdditionalIps($this->getAdditionalIP($client))
                    ->setAdditionalIpsNumber($client->getAdditionalIPAddress());
        }


        $createModel = new OrderCreate();
        $createModel->setModule('Server')
                ->setClient($clientID)
                ->setAutoAccept($client->getBaseFeatures()['autoAccept'])
                ->setCriteria($criteriaModel)
                ->setService($serviceModel);

        $api    = new EasyDCIM($client);
        $result = $api->order->create($createModel);

        CustomFields::set($this->params['params']['packageid'], $this->params['params']['serviceid'], 'OrderID', $result->id);
        if (!empty($result->service->related_id))
        {
            CustomFields::set($this->params['params']['packageid'], $this->params['params']['serviceid'], 'ServerID', $result->service->related_id);
            $device = (new Synchronize($this->params))->getDeviceInformation($result->service->related_id);
            Emails::sendServerCreateEmail($this->params['params']['packageid'], $this->params['params']['serviceid'], $device->metadata);
        }
    }

    /**
     * Calculate number of IP addresses 
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return string 
     * 
     */
    private function getAdditionalIP(IClient $client)
    {
        $additionalIP = $client->getAdditionalIPAddress();
        if ($additionalIP == 1)
        {
            return '/30';
        }
        $hosts = [];
        $mask  = null;
        for ($i = 1; $i <= 30; $i++)
        {
            $hosts[$i] = (1 << (32 - $i)) - 2;
        }
        for ($i = 30; $i >= 0; $i--)
        {
            if ($hosts[$i] >= $additionalIP)
            {
                $mask = $i;
                break;
            }
        }
        return '/' . $mask;
    }

    /**
     * Get Container of parts
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return $hddModel insteandof OrderPart
     * 
     */
    private function getParts(IClient $client)
    {
        $partContainer = new OrderPartContainer();
        if ($client->getHddModel() != 0 || $client->getHddSize() != 0)
        {
            $partContainer->setHdd($this->getHddModel($client));
        }
        if ($client->getSsdModel() != 0 || $client->getSsdSize() != 0)
        {
            $partContainer->setSsd($this->getSsdModel($client));
        }
        if ($client->getRamModel() != 0 || $client->getRamSize() != 0 )
        {
            $partContainer->setRam($this->getRamModel($client));
        }
        if ($client->getCpuModel() != 0 || $client->getCpuCores() != 0)
        {
            $partContainer->setCpu($this->getCpuModel($client));
        }
        return $partContainer;
    }

    /*
     * 
     * Get configurable options without disabled
     * 
     * @return array $options
     */

    private function getCustomInformation()
    {
        $options = ConfigurableOptions::getAll($this->params['params']['serviceid']);
        foreach ($options as $key => $value)
        {
            if (in_array($key, $this->disabledOptions))
            {
                unset($options[$key]);
            }
        }
        return $options;
    }

    /**
     * Check Active Parts
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return integer
     * 
     */
    private function checkPartIsActive(IClient $client)
    {


        if ($client->getHddModel() != 0 || $client->getSsdModel() != 0 ||
            $client->getRamModel() != 0 || $client->getCpuModel() != 0 ||
            $client->getHddSize() != 0 || $client->getSsdSize() != 0 ||
            $client->getRamSize() != 0 || $client->getCpuCores() != 0)
        {
            return 1;
        }
        return 0;
    }

    /**
     * Get HDD Part Model
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return $hddModel insteandof OrderPart
     * 
     */
    private function getHddModel(IClient $client)
    {
        $hddModel = new OrderPart();
        $hddModel->setPartType('hdd.size')
                ->setPartValue($client->getHddSize())
                ->setModel($client->getHddModel());

        return $hddModel;
    }

    /**
     * Get SSD Part Model
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return $hddModel insteandof OrderPart
     * 
     */
    private function getSsdModel(IClient $client)
    {
        $hddModel = new OrderPart();
        $hddModel->setPartType('ssd.size')
                ->setPartValue($client->getSsdSize())
                ->setModel($client->getSsdModel());

        return $hddModel;
    }

    /**
     * Get RAM Part Model
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return $hddModel insteandof OrderPart
     * 
     */
    private function getRamModel(IClient $client)
    {
        $hddModel = new OrderPart();
        $hddModel->setPartType('ram.size')
                ->setPartValue($client->getRamSize())
                ->setModel($client->getRamModel());

        return $hddModel;
    }

    /**
     * Get CPU Part Model
     * 
     * $param $client insteandof ClientAdapter
     * 
     * @return $hddModel insteandof OrderPart
     * 
     */
    private function getCpuModel(IClient $client)
    {
        $hddModel = new OrderPart();
        $hddModel->setPartType('cpu.cores')
                ->setPartValue($client->getCpuCores())
                ->setModel($client->getCpuModel());

        return $hddModel;
    }

    /**
     * Get Client ID
     * 
     * $param $client insteandof ClientAdapter
     * $return integer 
     * 
     */
    private function getClientID(IClient $client)
    {
        $check = $this->checkUserExists($client);
        if ($check->count == 1)
        {
            return $check->user->id;
        }
        $newUser = $this->createClient($client);
        return $newUser->id;
    }

    /**
     * Check user exsists on Easy DCIM 
     * 
     * $param $client insteandof ClientAdapter
     * $return JSON object 
     * 
     */
    private function checkUserExists(IClient $client)
    {
        $userDetails = new UserDetails();
        $userDetails->setEmail($client->getEmail());

        $api = new EasyDCIM($client);
        return $api->user->checkIfExists($userDetails);
    }

    /**
     * Create new EASY DCIM user
     * 
     * $param $client insteandof ClientAdapter
     * $return JSON object 
     * 
     */
    private function createClient(IClient $client)
    {

        $userDetails = new UserDetails();
        $userDetails->setUsername($client->getUsername())
                ->setFirstname($client->getFirstName())
                ->setLastname($client->getLastName())
                ->setEmail($client->getEmail())
                ->setPassword($client->getPassword())
                ->setPasswordConfirmation($client->getPassword())
                ->setCity($client->getCity())
                ->setAddress1($client->getAddress1())
                ->setAddress2($client->getAddress2())
                ->setState($client->getState())
                ->setCountryCode($client->getCountryCode())
                ->setPostcode($client->getPostCode());

        $createUser = new UserContainer();
        $createUser->setData($userDetails)
                ->setRole($this->getUserRole());

        $api = new EasyDCIM($client);
        return $api->user->create($createUser);
    }

    /**
     * Get user role 
     * 
     * $return integer
     * 
     */
    private function getUserRole()
    {
        $filePath = FileDir::getConfigDir() . DS . 'configuration.json';
        $file     = new FileReader($filePath);
        $config   = $file->read();
        return ($config->base->userRole) ?: 1;
    }

}
