<?php

namespace MGModule\EasyDCIM\Repositories\Actions;

use Carbon\Carbon;
use Exception;
use MGModule\EasyDCIM\Helpers\AdvancedBilling;
use MGModule\EasyDCIM\Helpers\CustomFields;
use MGModule\EasyDCIM\Helpers\InvoicingDate;
use MGModule\EasyDCIM\Helpers\Logs;
use MGModule\EasyDCIM\Helpers\ProductConfiguration;
use MGModule\EasyDCIM\MgLibs\Database\Models\Hostings;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Devices\Bandwidth;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;

/**
 * Description of Suspend
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class UsageUpdate extends AbstractRepository
{
    /*
     * Update Usage action
     * 
     */

    public function update()
    {
        //check Advaced Billing is active
        if (AdvancedBilling::isActive())
        {
            return;
        }
        $services = $this->getAllActiveServices();

        foreach ($services as $service)
        {
            if (empty(CustomFields::get($service->packageid, $service->id, 'ServerID')))
            {
                continue;
            }
            try
            {
                $this->updateRecord($service);
            }
            catch (Exception $ex)
            {
                Logs::save('UsageUpdate', $service, 'Service ID #: ' . $service->id . ' Message: ' . $ex->getMessage(), $service->packageid);
                echo print_r('Service ID #: ' . $service->id . ' Message: ' . $ex->getMessage(), true);
            }
        }
    }

    /*
     * Update all service assigned to server
     * 
     * 
     * @return object insteandof Hostings
     */

    private function getAllActiveServices()
    {
        return Hostings::where([
                    ['server', $this->params['params']['serverid']],
                    ['domainstatus', 'Active'],
                ])->get();
    }

    /*
     * Update Service
     * 
     * @param object $service insteandof Hostings
     * 
     */

    private function updateRecord($service)
    {
        $serverID = CustomFields::get($service->packageid, $service->id, 'ServerID');
        if (CustomFields::get($service->packageid, $service->id, 'bandwidthUsageFromRegistrationDate') == "on")
        {
            $startDate = InvoicingDate::checkAndGetDate($service->id) . " 00:00:01";
        }
        else
        {
            $startDate = InvoicingDate::getFirstDay() . " 00:00:01";
        }

        $endDate   = Carbon::now()->toDateTimeString();
        $bandwidth = $this->getBandwidth($serverID, $startDate, $endDate);

        if ($bandwidth)
        {
            //Save Service details
            $service->bwlimit    = $this->getBandwidthLimit($service);
            $service->bwusage    = $bandwidth->{'BW_TOTAL'};
            $service->lastupdate = Carbon::now()->toDateTimeString();
            $service->save();
            //Save Percentile custom field
            $percentilUsage      = $bandwidth->{'95TH_PERC'} / 1000;
            CustomFields::set($service->packageid, $service->id, 'percentileUsage', $percentilUsage);
        }
    }

    /*
     * Get Bandwidth information from Easy DCIM
     * 
     * @param object $service insteandof Hostings, date $startDate, $endDate
     * 
     * @return JSON object
     * 
     */

    private function getBandwidth($serverID, $startDate, $endDate)
    {
        try
        {
            $bandwidthModel = new Bandwidth();
            $bandwidthModel->setStartDate($startDate)
                    ->setEndDate($endDate)
                    ->setUnits('MB');

            $api = new EasyDCIM(new ClientAdapter($this->params));
            return $api->device->bandwidth($bandwidthModel, $serverID);
        }
        catch (\Exception $ex)
        {
            echo print_r('API Error: ' . $ex->getMessage(), true);
        }
    }

    /*
     * Get Bandwidth information from WHMCS
     * 
     * @param object $service insteandof Hostings
     * 
     * @return integer
     * 
     */

    private function getBandwidthLimit($service)
    {
        $productBandwidth = ProductConfiguration::getOne($service->packageid, 'bandwidth');   //   = $this->getProduct();
        $customBandwidth  = CustomFields::get($service->packageid, $service->id, 'bandwidth');
        $totalBandwidth   = ($customBandwidth) ?: $productBandwidth;

        if ($totalBandwidth > 0)
        {
            $totalBandwidth *= 1024;
        }
        return $totalBandwidth;
    }

}
