<?php

namespace MGModule\EasyDCIM\Repositories\ClientDetails;

use MGModule\EasyDCIM\Helpers\ProductConfiguration;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Interfaces\IClient;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Graphs\Graph;

/**
 * Description of Install
 *
 * @author Mateusz Pawłowski
 */
class ClientDetails extends AbstractRepository
{

    private $configuration;

    /**
     * Get information for Client Area main page
     *
     * @return array
     */
    public function getData()
    {
        $this->setConfiguration();
        $device = $this->getDeviceInformation();

        $fields['buttons'] = $this->getButtons();
        $fields['details'] = $this->getDetails($device);
        $fields['graphs']  = $this->getGraphs();

        return $fields;
    }

    public function downloadKey(){
        $device = $this->getDeviceInformation();
        ob_clean();
        header('Content-Disposition: attachment; filename="' . mktime() . '.ppk"');
        echo trim((string) $device->metadata->{'SSH Private Key'});
        die();

    }
    /**
     * Set Mg Product Configuration
     *
     * @return string
     */
    private function setConfiguration()
    {
        $this->configuration = ProductConfiguration::get($this->params['params']['packageid']);
    }

    /**
     * Build Login Redirect Link
     *
     * @return string
     */
    private function buildLoginLink()
    {
        $params = [
            'action'    => 'productdetails',
            'id'        => $this->params['id'],
            'mg-action' => 'getLoginLink'
        ];
        return 'clientarea.php?' . http_build_query($params);
    }

    private function buildDownloadKeyLink()
    {
        $params = [
            'action'    => 'productdetails',
            'id'        => $this->params['id'],
            'mg-action' => 'downloadKey'
        ];
        return 'clientarea.php?' . http_build_query($params);
    }

    /**
     * Build Console Redirect Link
     *
     * @return string
     */
    private function buildConsoleLink()
    {
        $params = [
            'action'    => 'productdetails',
            'id'        => $this->params['id'],
            'mg-action' => 'getKVMConsole'
        ];
        return 'clientarea.php?' . http_build_query($params);
    }

    /**
     * Get Order information from Easy DCIM
     *
     * @return JSON object
     */
    private function getOrderInformation(IClient $client)
    {
        $api = new EasyDCIM($client);
        return $api->device->getInformation();
    }

    /**
     * Get Device information from Easy DCIM
     *
     * @return JSON object
     */
    private function getDeviceInformation()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->device->getInformation();
    }

    /*
     * Get Control Panel Section
     * 
     * @return array $buttons
     */

    private function getButtons()
    {
        $buttons = [];
        if (in_array('caBoot', $this->configuration['caServerActions']))
        {
            $buttons['powerOn'] = true;
        }
        if (in_array('caReboot', $this->configuration['caServerActions']))
        {
            $buttons['reboot'] = true;
        }
        if (in_array('caShutdown', $this->configuration['caServerActions']))
        {
            $buttons['powerOff'] = true;
        }
        if (in_array('caColdReset', $this->configuration['caServerActions']))
        {
            $buttons['coldReset'] = true;
        }
        if (in_array('caSso', $this->configuration['caServerActions']))
        {
            $buttons['loginLink'] = $this->buildLoginLink();
        }
        if (in_array('caKvmConsole', $this->configuration['caServerActions']))
        {
            $buttons['kvmConsole'] = $this->buildConsoleLink();
        }
        if (in_array('caNoVNCKvmConsole', $this->configuration['caServerActions']))
        {
            $buttons['noVNCKvmConsole'] = true;
        }



        return $buttons;
    }

    /*
     * Get Details Section
     * 
     * $param JSON object $device
     * @return array $details
     */

    private function getDetails($device)
    {
        $details = [];
        if (in_array('caModel', $this->configuration['caBaseFeatures']))
        {
            $details['model'] = $device->model;
        }
        if (in_array('caLabel', $this->configuration['caBaseFeatures']))
        {
            $details['label'] = $device->label;
        }
        if (in_array('caStatus', $this->configuration['caBaseFeatures']))
        {
            $details['deviceStatus'] = ucfirst($device->device_status);
        }
        if (in_array('caSerialnumber', $this->configuration['caBaseFeatures']))
        {
            $details['serial'] = $device->serialnumber1;
        }
        if (in_array('caPurchaseDate', $this->configuration['caBaseFeatures']))
        {
            $details['purchaseDate'] = $device->purchase_date;
        }
        if (in_array('caWarrantyMonths', $this->configuration['caBaseFeatures']))
        {
            $details['warrantyMonths'] = $device->warranty_months;
        }
        if (in_array('caHostname', $this->configuration['caBaseFeatures']))
        {
            $details['ipaddress'] = $device->metadata->{'IP Address'};
            $details['hostname']  = $device->metadata->Hostname;
        }
        if (in_array('caChangeHostname', $this->configuration['caBaseFeatures']))
        {
            $details['changeHostname'] = true;
        }
        if (in_array('caIps', $this->configuration['caBaseFeatures']))
        {
            $details['ipaddresses'] = $device->ip_addresses;
        }
        if (in_array('caLocation', $this->configuration['caBaseFeatures']))
        {
            $details['location'] = $device->location->name;
        }
        if (in_array('caRack', $this->configuration['caBaseFeatures']))
        {
            $details['rack'] = $device->labeledRackWithPosition;
        }

        if (in_array('caOs', $this->configuration['caBaseFeatures']))
        {
            $details['os'] = $device->metalist->OS;
        }

        if (in_array('caOsInstallation', $this->configuration['caBaseFeatures']))
        {
            $details['osInstallation'] = $device->os_installation;
        }

        if (in_array('caSshDetails', $this->configuration['caBaseFeatures']))
        {
            $details['sshUser'] = $device->metadata->{'SSH Username'};
            $details['sshPassword'] = $device->metadata->{'SSH Password'};
            $details['sshPasswordHash'] = str_repeat('*', strlen($device->metadata->{'SSH Password'}));
            $details['sshPort'] = $device->metadata->{'SSH Port'};
            $details['sshPrivateKey'] = trim($device->metadata->{'SSH Private Key'});
            $details['sshPrivateKeyUrl'] = $this->buildDownloadKeyLink();
        }

        return $details;
    }

    /*
     * Get Graphs Section
     * 
     * @return boolean
     */

    private function getGraphs()
    {
        $graphRepository = new Graph($this->params);
        $graphs          = [];
        if (!empty($graphRepository->getCAAvailableGraphsList()))
        {
            $maxDate     = $graphRepository->getDateMax();
            $displayDate = $graphRepository->getDisplayDate();

            $graphs = array_merge($maxDate, $displayDate);
        }
        return $graphs;
    }

}
