<?php

namespace MGModule\EasyDCIM\Repositories\Configuration;

use MGModule\EasyDCIM\Helpers as Helper;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Configuration\Traits\TraitFieldsOption;

/**
 * Description of Configuration
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class ConfigurableOptions extends AbstractRepository
{

    use TraitFieldsOption;

    /*
     * Create Configurable Options
     * @return array
     * 
     */

    public function create()
    {
        $this->getServerDetails();
        $this->createConfigurableOptions();
        return ['success' => Lang::T('configurableOptionsCreated')];
    }

    /*
     * Get Configurable options table
     * 
     * @return array
     */

    public function get()
    {
        $this->getServerDetails();
        Helper\ConfigurableOptions::checkExcistsOptionsGroup($this->params['id']);
        return ['table' => $this->getHTMLTable()
        ];
    }

    /*
     * Get Configuration table
     * 
     * @return string
     */

    public function getHTMLTable()
    {
        $data = $this->getAllOptinosArray();

        $rows = "";
        foreach ($data as $oneRow)
        {
            $oneRow['type'] = Helper\ConfigurableOptions::typeAdapter($oneRow['type']);
            $rows           .= $this->tableParseRow('configurableOptionsRow', $oneRow);
        }
        return $rows;
    }

    /*
     * Prepare array with all configurable options
     * 
     * @return array
     */

    private function getAllOptinosArray()
    {
        return [
            $this->getModelOptionsArray(),
            $this->getLocationOptionsArray(),
            $this->getOsTemplateOptionsArray(),
            $this->getServiceAccessOptionsArray(),
            $this->getBandwidthArray(),
            $this->getAdditionalIPArray(),
            $this->getHDDSizeArray(),
            $this->getSSDSizeArray(),
            $this->getRAMSizeArray(),
            $this->getCPUCoresArray()
        ];
    }

    /*
     * Prepare model configurable options array
     * 
     * @return array
     */

    private function getModelOptionsArray()
    {
        $options = [];
        foreach ($this->getServerModelOpions() as $key => $value)
        {
            $options[] = $key . '|' . $value;
        }

        return[
            'name'    => 'model|Model',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare service access level configurable options array
     * 
     */

    private function getServiceAccessOptionsArray()
    {
        $options = [];
        foreach ($this->getAccessLevelOption() as $key => $value)
        {
            $options[] = $key . '|' . $value;
        }

        return[
            'name'    => 'serviceAccess|Service Access',
            'type'    => 1,
            'hidden'  => 1,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare location configurable options array
     * 
     * @return array
     */

    private function getLocationOptionsArray()
    {
        $options = [];
        foreach ($this->getLocationsOpions() as $key => $value)
        {
            $options[] = $key . '|' . $value;
        }

        return[
            'name'    => 'location|Location',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare os template configurable options array
     * 
     * @return array
     */

    private function getOsTemplateOptionsArray()
    {
        foreach ($this->getOSTemplateOptions()['admin'] as $key => $value)
        {
            $options[] = $value['text'] . '|' . $value['text'];
        }
        return[
            'name'    => 'template|OS Template',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare bandwidth configurable options array
     * 
     * @return array
     */

    private function getBandwidthArray()
    {
        return[
            'name'    => 'bandwidth|Bandwidth',
            'type'    => 4,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 1000,
            'options' => ['1|GB']
        ];
    }

    /*
     * Prepare additional ip configurable options array
     * 
     * @return array
     */

    private function getAdditionalIPArray()
    {
        return[
            'name'    => 'additionalIpsNumber|Additional IP Addresses',
            'type'    => 4,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 62,
            'options' => ['1|[Unit]']
        ];
    }

    /*
     * Prepare hdd size configurable options array
     * 
     * @return array
     */

    private function getHDDSizeArray()
    {
        $options = [
            '10240-20480|10240-20480 [MB]',
            '40960-61440|40960-61440 [MB]',
        ];
        return[
            'name'    => 'hdd|HDD Size',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare ssd size configurable options array
     * 
     * @return array
     */

    private function getSSDSizeArray()
    {
        $options = [
            '10240-20480|10240-20480 [MB]',
            '40960-61440|40960-61440 [MB]',
        ];
        return[
            'name'    => 'ssd|SSD Size',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare ram size configurable options array
     * 
     * @return array
     */

    private function getRAMSizeArray()
    {
        $options = [
            '10240-20480|10240-20480 [MB]',
            '40960-61440|40960-61440 [MB]',
        ];
        return[
            'name'    => 'ram|RAM Size',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Prepare cpu cores configurable options array
     * 
     * @return array
     */

    private function getCPUCoresArray()
    {
        $options = [
            '1-2|1-2 [Cores] ',
            '3-6|3-6 [Cores] ',
        ];
        return[
            'name'    => 'cpuCores|CPU Cores',
            'type'    => 1,
            'hidden'  => 0,
            'min'     => 0,
            'max'     => 0,
            'options' => $options
        ];
    }

    /*
     * Create Configurable Options with ConfigurableOptions
     * 
     * @return array 
     */

    public function createConfigurableOptions()
    {
        $optionGroup = Helper\ConfigurableOptions::prepareConfigurableOptions($this->params['id']);
        $options     = $this->getAllOptinosArray();
        foreach ($options as $option)
        {
            $this->createConfigurableSubAndOptions($optionGroup, $option);
        }
    }

    /*
     * Create Option and sub-options to configurable group
     * 
     * @params object $optionGroup insteandof ProductConfigGroups array $option
     */

    private function createConfigurableSubAndOptions($optionGroup, $option)
    {

        $options = Helper\ConfigurableOptions::createOptions($optionGroup, $option['name'], $option['type'], $option['min'], $option['max'], 0, $option['hidden']);

        foreach ($option['options'] as $value)
        {
            Helper\ConfigurableOptions::createSubOptions($options->id, $value);
        }
    }

}
