<?php

namespace MGModule\EasyDCIM\Repositories\Configuration;

use Exception;
use MGModule\EasyDCIM\Helpers\AdvancedBilling;
use MGModule\EasyDCIM\Helpers\CustomFields;
use MGModule\EasyDCIM\Helpers\FileDir;
use MGModule\EasyDCIM\Helpers\ProductConfiguration;
use MGModule\EasyDCIM\MgLibs\Forms\ButtonField;
use MGModule\EasyDCIM\MgLibs\Forms\CheckboxField;
use MGModule\EasyDCIM\MgLibs\Forms\Creator;
use MGModule\EasyDCIM\MgLibs\Forms\InfoField;
use MGModule\EasyDCIM\MgLibs\Forms\NumberField;
use MGModule\EasyDCIM\MgLibs\Forms\SelectField;
use MGModule\EasyDCIM\MgLibs\Forms\TextField;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Configuration\Traits\TraitFieldsOption;
use MGModule\EasyDCIM\Repositories\Migrations\Migrations;
use MGModule\EasyDCIM\Repositories\Modules\Traits\TraitModules;
use MGModule\EasyDCIM\Server;
use const DS;

/**
 * Description of Configuration
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class Configuration extends AbstractRepository
{

    use TraitFieldsOption,
        TraitModules;

    /**
     * Check and get server details
     * Create custom fields required to correctly working module
     * 
     * @return array
     * @throw Exception();
     */
    public function getForm()
    {
        $this->checkServerAssign();
        $this->createCustomFields();
        $this->checkMigartion();
        return $this->buildForm();
    }

    /**
     * Check if the server is selected.
     * 
     * @throw Exception();
     */
    private function checkServerAssign()
    {
        if ($this->params['servergroup'] == 0)
        {
            throw new Exception(Lang::T('emptyServerGroup'));
        }
        $this->getServerDetails();
    }

    private function getActionButtonParams()
    {
        return [
            'action'      => $this->params['action'],
            'module'      => $this->params['module'],
            'servergroup' => $this->params['servergroup'],
            'id'          => $this->params['id'],
            'mode'        => $this->params['mode'],
        ];
    }

    /**
     * Build form 
     * 
     * @return array
     */
    private function buildForm()
    {

        $fieldsValue = ProductConfiguration::get($this->params['id']);
        $form        = new Creator('custom');

        $form->addField('legend', 'DefaultOptions');

        $osModuleActive = $this->isModuleActive('os');
        if (!$osModuleActive)
        {
            $vars['warning'][] = Lang::T('osModuleIsNotActive');
        }

        if (!$this->isModuleActive('dns-manager'))
        {
            $vars['warning'][] = Lang::T('dnsModuleIsNotActive');
        }

        $field           = new InfoField();
        $field->name     = "cron";
        $field->text     = 'php -q ' . FileDir::getModuleDir() . DS . 'Cron' . DS . 'Cron.php';
        $field->colWidth = 8;
        $form->addField($field);

        $field                = new ButtonField();
        $field->name          = 'mgconfig[configurableOption]';
        $field->color         = "primary";
        $field->class         = "MGConfigurableOptionsModal";
        $field->dataAttr      = $this->getActionButtonParams();
        $field->enableContent = true;
        $field->enableLabel   = true;
        $field->textLabel     = true;
        $form->addField($field);

        $field                   = new SelectField();
        $field->name             = 'mgconfig[model]';
        $field->options          = $this->getServerModelOpions();
        $field->value            = $fieldsValue['model'];
        $field->id               = "mgconfigModel";
        $field->translateOptions = false;
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);

        $field                   = new SelectField();
        $field->name             = 'mgconfig[location]';
        $field->options          = $this->getLocationsOpions();
        $field->value            = $fieldsValue['location'];
        $field->id               = "mgconfigLocation";
        $field->translateOptions = false;
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);

        if ($osModuleActive)
        {
            $field                   = new SelectField();
            $field->name             = 'mgconfig[osTemplate]';
            $field->value            = $fieldsValue['osTemplate'];
            $field->id               = "mgconfigOsTemplate";
            $field->translateOptions = false;
            $field->colWidth         = 8;
            $field->translateOptions = false;
            $form->addField($field);
        }

        $field                  = new NumberField();
        $field->name            = 'mgconfig[bandwidth]';
        $field->value           = $fieldsValue['bandwidth'];
        $field->labelcolWidth   = 3;
        $field->colWidth        = 8;
        $field->descriptionHtml = '<div class="mgUnitStyle">[GB]</div>';
        $form->addField($field);

        $field           = new TextField();
        $field->name     = 'mgconfig[additionalIPAddress]';
        $field->value    = $fieldsValue['additionalIPAddress'];
        $field->colWidth = 8;
        $form->addField($field);

        $field                    = new TextField();
        $field->name              = 'mgconfig[HDDSize]';
        $field->value             = $fieldsValue['HDDSize'];
        $field->colWidth          = 8;
        $field->enableDescription = true;
        $field->descriptionHtml   = '<div class="mgUnitStyle">[MB]</div>';
        $form->addField($field);

        $field                    = new TextField();
        $field->name              = 'mgconfig[SSDSize]';
        $field->value             = $fieldsValue['SSDSize'];
        $field->colWidth          = 8;
        $field->enableDescription = true;
        $field->descriptionHtml   = '<div class="mgUnitStyle">[MB]</div>';
        $form->addField($field);

        $field                    = new TextField();
        $field->name              = 'mgconfig[RAMSize]';
        $field->value             = $fieldsValue['RAMSize'];
        $field->colWidth          = 8;
        $field->enableDescription = true;
        $field->descriptionHtml   = '<div class="mgUnitStyle">[MB]</div>';
        $form->addField($field);

        $field                    = new TextField();
        $field->name              = 'mgconfig[CPUCores]';
        $field->value             = $fieldsValue['CPUCores'];
        $field->colWidth          = 8;
        $field->enableDescription = true;
        $form->addField($field);


        $form->addField('legend', 'ProductConfiguration');

        $field                    = new CheckboxField();
        $field->name              = 'mgconfig[baseFeatures]';
        $field->options           = [
            'autoAccept'    => Lang::T('autoAccept'),
            "requirePdu"    => Lang::T('requirePdu'),
            "requireSwitch" => Lang::T('requireSwitch'),
            "debug"         => Lang::T('debug')
        ];
        $field->value             = $fieldsValue['baseFeatures'];
        $field->enableDescription = false;
        $field->labelcolWidth     = 3;
        $form->addField($field);

        $field                   = new SelectField();
        $field->name             = 'mgconfig[serviceAccessLevel]';
        $field->options          = $this->getAccessLevelOption();
        $field->value            = $fieldsValue['serviceAccessLevel'];
        $field->id               = "mgconfigServiceAccessLevel";
        $field->translateOptions = false;
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);

        $form->addField('legend', 'ItemsModel');

        $field                   = new SelectField();
        $field->name             = 'mgconfig[HDDModel]';
        $field->options          = $this->getHDDModelOpions();
        $field->value            = $fieldsValue['HDDModel'];
        $field->id               = "mgconfigHHDModel";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);

        $field                   = new SelectField();
        $field->name             = 'mgconfig[SSDModel]';
        $field->options          = $this->getSSDModelOpions();
        $field->value            = $fieldsValue['SSDModel'];
        $field->id               = "mgconfigSSDModel";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);

        $field                   = new SelectField();
        $field->name             = 'mgconfig[RAMModel]';
        $field->options          = $this->getRAMModelOpions();
        $field->value            = $fieldsValue['RAMModel'];
        $field->id               = "mgconfigRAMModel";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);


        $field                   = new SelectField();
        $field->name             = 'mgconfig[CPUModel]';
        $field->options          = $this->getCPUModelOpions();
        $field->value            = $fieldsValue['CPUModel'];
        $field->id               = "mgconfigCPUCores";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);



        $form->addField('legend', 'ClientAreaFeatures');


        $field                = new CheckboxField();
        $field->name          = 'mgconfig[caBaseFeatures]';
        $field->options       = [
            'caImage'          => Lang::T('caImage'),
            'caModel'          => Lang::T('caModel'),
            'caLabel'          => Lang::T('caLabel'),
            'caStatus'         => Lang::T('caStatus'),
            'caSerialnumber'   => Lang::T('caSerialnumber'),
            'caPurchaseDate'   => Lang::T('caPurchaseDate'),
            'caWarrantyMonths' => Lang::T('caWarrantyMonths'),
            'caHostname'       => Lang::T('caHostname'),
            'caChangeHostname' => Lang::T('caChangeHostname'),
            'caIps'            => Lang::T('caIps'),
            'caLocation'       => Lang::T('caLocation'),
            'caRack'           => Lang::T('caRack'),
            'caGraphs'         => Lang::T('caGraphs'),
            'caSshDetails'     => Lang::T('caSshDetails'),
            'caOs'             => Lang::T('caOs'),
            'caOsInstallation' => Lang::T('caOsInstallation'),
        ];
        $field->value         = $fieldsValue['caBaseFeatures'];
        $field->selectAll     = true;
        $field->labelcolWidth = 3;
        $field->colWidth      = 8;
        $form->addField($field);



        $field                    = new CheckboxField();
        $field->name              = 'mgconfig[caSelectedGraphs]';
        $field->options           = $this->getGraphsOptions();
        $field->value             = $fieldsValue['caSelectedGraphs'];
        $field->enableDescription = true;
        $field->selectAll         = true;
        $field->labelcolWidth     = 3;
        $form->addField($field);



        $field                = new CheckboxField();
        $field->name          = 'mgconfig[caServerActions]';
        $field->options       = [
            'caReboot'          => Lang::T('caReboot'),
            'caBoot'            => Lang::T('caBoot'),
            'caShutdown'        => Lang::T('caShutdown'),
            'caColdReset'       => Lang::T('caColdReset'),
            'caSso'             => Lang::T('caSso'),
            'caKvmConsole'      => Lang::T('caKvmConsole'),
            'caNoVNCKvmConsole' => Lang::T('caNoVNCKvmConsole')
        ];
        $field->value         = $fieldsValue['caServerActions'];
        $field->selectAll     = true;
        $field->labelcolWidth = 3;
        $form->addField($field);

        $field                    = new CheckboxField();
        $field->name              = 'mgconfig[modules]';
        $field->options           = [
            'usageCollector' => Lang::T('usageCollector'),
            'clientTraffic'  => Lang::T('clientTraffic'),
            'osInstallation' => Lang::T('osInstallation'),
            'dnsManager'     => Lang::T('dnsManager'),
            'passwordManagement' => Lang::T('passwordManagement'),
        ];
        $field->value             = $fieldsValue['modules'];
        $field->selectAll         = true;
        $field->enableDescription = true;
        $field->labelcolWidth     = 3;
        $form->addField($field);

        $field                = new CheckboxField();
        $field->name          = 'mgconfig[caSidebarActions]';
        $field->options       = [
            'changePassword'    => Lang::T('changePassword'),
            'serverInformation' => Lang::T('serverInformation'),
        ];
        $field->value         = $fieldsValue['caSidebarActions'];
        $field->selectAll     = true;
        $field->labelcolWidth = 3;
        $form->addField($field);
        //OS Installation Templates

        if ($osModuleActive)
        {
            $field                    = new SelectField();
            $field->name              = 'mgconfig[caOsTemplate]';
            $field->value             = $fieldsValue['caOsTemplate'];
            $field->id                = "mgconfigCaOsTemplate";
            $field->colWidth          = 8;
            $field->translateOptions  = false;
            $field->multiple          = true;
            $field->enableDescription = true;
            $form->addField($field);
        }



        $form->addField('legend', 'ServiceActions');

        $field                   = new CheckboxField();
        $field->name             = 'mgconfig[suspendActions]';
        $field->options          = ['send_mail' => true];
        $field->value            = $fieldsValue['suspendActions'];
        $field->labelcolWidth    = 3;
        $form->addField($field);
        //Template Suspend
        $field                   = new SelectField();
        $field->name             = 'mgconfig[templateSuspend]';
        $field->options          = $this->getEmailTempaltes('admin');
        $field->value            = ($fieldsValue['templateSuspend']) ?: $this->getEmailTemplateID('EasyDCIM Service Suspension');
        $field->id               = "mgconfigTemplateSuspend";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);


        $field                = new CheckboxField();
        $field->name          = 'mgconfig[unsuspendActions]';
        $field->options       = ['send_mail' => true];
        $field->value         = $fieldsValue['unsuspendActions'];
        $field->labelcolWidth = 3;
        $form->addField($field);

        //Template Unsuspend
        $field                   = new SelectField();
        $field->name             = 'mgconfig[templateUnsuspend]';
        $field->options          = $this->getEmailTempaltes('admin');
        $field->value            = ($fieldsValue['templateUnsuspend']) ?: $this->getEmailTemplateID('EasyDCIM Service Unsuspension');
        $field->id               = "mgconfigTemplateUnsuspend";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);



        $field                   = new CheckboxField();
        $field->name             = 'mgconfig[terminateActions]';
        $field->options          = ['send_mail' => true];
        $field->value            = $fieldsValue['terminateActions'];
        $field->labelcolWidth    = 3;
        $form->addField($field);
        //templateUnsuspend
        $field                   = new SelectField();
        $field->name             = 'mgconfig[templateTerminate]';
        $field->options          = $this->getEmailTempaltes('admin');
        $field->value            = ($fieldsValue['templateTerminate']) ?: $this->getEmailTemplateID('EasyDCIM Service Termination');
        $field->id               = "mgconfigTemplateTerminate";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);


        $field                   = new SelectField();
        $field->name             = 'mgconfig[notificationAdminId]';
        $field->options          = $this->getAdmins();
        $field->value            = $fieldsValue['notificationAdminId'];
        $field->id               = "mgconfigNotificationAdminId";
        $field->translateOptions = false;
        $field->colWidth         = 8;
        $field->multiple         = true;
        $field->translateOptions = false;
        $form->addField($field);


        $form->addField('legend', 'ServiceNotification');
        $field                   = new SelectField();
        $field->name             = 'mgconfig[notificationServerCreate]';
        $field->options          = $this->getEmailTempaltes('product');
        $field->value            = ($fieldsValue['notificationServerCreate']) ?: $this->getEmailTemplateID('EasyDCIM - Welcome');
        $field->id               = "mgconfigNotificationServerCreate";
        $field->colWidth         = 8;
        $field->translateOptions = false;
        $form->addField($field);


        $vars['form'] = $form->getHTML('custom');
        return $vars;
    }

    /**
     * Add libs to header
     * 
     * @return template
     */
    public function loadLibs()
    {
        Server::I()->isAdmin(true);
        $vars = [
            'assetsURL'   => Server::I()->getAssetsURL(),
            'mainJSONURL' => 'configproducts.php',
        ];
        //Check is Advanced Billling is active
        if (!AdvancedBilling::isActive())
        {
            $vars['percentileCost'] = (ProductConfiguration::getOne($_REQUEST['id'], 'percentileCost')) ?: '0.000';
            $vars['percentile']     = (ProductConfiguration::getOne($_REQUEST['id'], 'percentileSoftLimit')) ?: '0';
        }
        return $this->parseTemplateToString('configurationHead', '/pages/configuration', $vars);
    }

    /**
     * Save Fields 
     * 
     * @throw database error
     */
    public function saveFields($vars = [])
    {
        if (!is_array($_REQUEST['mgconfig']))
        {
            return;
        }

        ProductConfiguration::set($vars['pid'], $_REQUEST['mgconfig']);
    }

    /**
     * Create custom fields with CustomFields
     * 
     */
    protected function createCustomFields()
    {
        CustomFields::create($this->params['id'], 'OrderID|Order ID');
        CustomFields::create($this->params['id'], 'ServerID|Server ID');
        CustomFields::create($this->params['id'], 'percentileUsage|Percentile Usage');
        CustomFields::create($this->params['id'], 'bandwidthUsageFromRegistrationDate|Bandwidth Usage From Registration Date', 'tickbox');
    }

    /*
     * Get Os Template ListF
     * 
     * @return array
     */

    public function getOsTemplateList()
    {
        $this->checkServerAssign();
        return $this->getOSTemplateOptions();
    }

    /*
     * Check Is Available old version 
     * 
     */

    private function checkMigartion()
    {
        $migration = new Migrations($this->params);
        $migration->check();
    }

}
