<?php

namespace MGModule\EasyDCIM\Repositories\Configuration\Traits;

use Exception;
use MGModule\EasyDCIM\Helpers\FileDir;
use MGModule\EasyDCIM\Helpers\ProductConfiguration;
use MGModule\EasyDCIM\MgLibs\Database\Models\Admins;
use MGModule\EasyDCIM\MgLibs\Database\Models\EmailTemplates;
use MGModule\EasyDCIM\MgLibs\Database\Models\ServersRelations;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Models\ListWithType;
use MGModule\EasyDCIM\MgLibs\FileReader\FileReader;
use MGModule\EasyDCIM\MgLibs\Lang;

/**
 * Description of TraitModel
 *
 * @author Mateusz Pawłowski
 */
trait TraitFieldsOption
{

    /**
     * Get server details assigned to product
     *
     * @throw databse error
     */
    protected function getServerDetails()
    {
        $server                 = ServersRelations::where('groupid', $this->params['servergroup'])->with('servers')->first();
        $this->params['params'] = [
            'serverip'       => $server->servers->ipaddress,
            'serverhostname' => $server->servers->hostname,
            'serversecure'   => $server->servers->secure,
            'serverpassword' => decrypt($server->servers->password),
        ];
    }

    /**
     * Prepare array to select fields
     *
     * $params object $object string $name, $type
     * @return array $options
     */
    private function prepareArray($object, $name = 'name', $type = null)
    {
        $opions = [];
        foreach ($object as $value)
        {
            if (!is_null($type) && $value->type != $type)
            {
                continue;
            }

            $opions[$value->id] = $value->{$name};
        }
        return $opions;
    }

    /**
     * Get Model from Easy DCIM
     *
     * @param string $type
     * @return JSON object
     */
    private function getModels($type)
    {
        $api   = new EasyDCIM(new ClientAdapter($this->params));
        $model = new ListWithType();
        $model->setType($type);
        return $api->models->listModelsWithType($model);
    }

    /**
     * Get Server Model Options
     *
     * @return array
     */
    protected function getServerModelOpions()
    {
        $api   = new EasyDCIM(new ClientAdapter($this->params));

        $list = $api->models->listServerModels();

        if (is_null($list))
        {
            return[Lang::T('NoModelOptions')];
        }
        return $this->prepareArray($list);
    }

    /**
     * Get HDD Model Options
     *
     * @return array
     */
    protected function getHDDModelOpions()
    {
        $list = $this->getModels('HDD');
        if (is_null($list))
        {
            return[Lang::T('NoModelOptions')];
        }
        $list    = $this->prepareArray($list);
        $list[0] = Lang::T('unassigned');
        return $list;
    }

    /**
     * Get SSD Model Options
     *
     * @return array
     */
    protected function getSSDModelOpions()
    {
        $list = $this->getModels('SSD');
        if (is_null($list))
        {
            return[Lang::T('NoModelOptions')];
        }
        $list    = $this->prepareArray($list);
        $list[0] = Lang::T('unassigned');
        return $list;
    }

    /**
     * Get RAM Model Options
     *
     * @return array
     */
    protected function getRAMModelOpions()
    {
        $list = $this->getModels('RAM');
        if (is_null($list))
        {
            return[Lang::T('NoModelOptions')];
        }
        $list    = $this->prepareArray($list);
        $list[0] = Lang::T('unassigned');
        return $list;
    }

    /**
     * Get RAM Model Options
     *
     * @return array
     */
    protected function getCPUModelOpions()
    {
        $list = $this->getModels('CPU');
        if (is_null($list))
        {
            return[Lang::T('NoModelOptions')];
        }
        $list    = $this->prepareArray($list);
        $list[0] = Lang::T('unassigned');
        return $list;
    }

    /**
     * Get Locations from Easy DCIM
     *
     * @return JSON object
     */
    private function getLocations()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->locations->getLists();
    }

    /**
     * Get Locations options
     *
     * @return array
     */
    protected function getLocationsOpions()
    {
        $locationList = $this->getLocations();
        if (is_null($locationList))
        {
            return[Lang::T('NoLocationOptions')];
        }
        return $this->prepareArray($locationList);
    }

    /**
     * Get Access Level from Easy DCIM
     *
     * @return JSON object
     */
    private function getAccessLevel()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->service->getAccessLevelList();
    }

    /**
     * Get Access Level options
     *
     * @return array
     */
    protected function getAccessLevelOption()
    {
        $accessLevelList = $this->getAccessLevel();
        if (is_null($accessLevelList))
        {
            return[Lang::T('NoAccessLevelOptions')];
        }
        return $this->prepareArray($accessLevelList, 'name', 'Server');
    }

    /**
     * Get OS Templates List from Easy DCIM
     *
     * @return JSON object
     */
    private function getOSTemplates()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->os->getTemplateList();
    }

    /*
     * Get templates for location 
     *
     */

    private function getDeviceAssignedToLocation($locationID = null)
    {
        try
        {
            $locationID = ($locationID) ?: $this->params['locationID'];
            $api        = new EasyDCIM(new ClientAdapter($this->params));
            return $api->os->getOsTemplateForLocation($locationID);
        }
        catch (Exception $ex)
        {
            if ($ex->getMessage() == "Unable to find appropriate server. ")
            {
                throw new Exception(Lang::absoluteT('apiError', 'noAvailableTemplates'));
            }
            throw $ex;
        }
    }

    protected function getAvailableOSTemplateList($locationID = null)
    {
        $device = $this->getDeviceAssignedToLocation($locationID);
        $list   = $this->getOSTemplates();

        if (is_null($list))
        {
            return;
        }
        $options = [];
        foreach ($list as $value)
        {
            if ($value->server_id != $device->id)
            {
                continue;
            }
            $options[] = $value;
        }
        return $options;
    }

    /**
     * Get Access Level options
     *
     * @return array
     */
    protected function getOSTemplateOptions()
    {
        $list    = $this->getAvailableOSTemplateList();
        $options = [];
        $config  = ProductConfiguration::get($this->params['id']);
        foreach ($list as $value)
        {
            $options['client'][] = [
                'id'       => $value->name,
                'text'     => $value->name,
                'selected' => in_array($value->name, $config['caOsTemplate'])
            ];
            $options['admin'][]  = [
                'id'       => $value->name,
                'text'     => $value->name,
                'selected' => ($value->name == $config['osTemplate'])
            ];
        }
        return $options;
    }

    /**
     * Get all admin email templates for WHMCS
     *
     * @return array
     */
    protected function getEmailTempaltes($type)
    {
        $emailTemplates = EmailTemplates::where('type', $type)->get();
        return $this->prepareArray($emailTemplates);
    }

    /**
     * Get all WHMCS admins
     *
     * @return array
     */
    protected function getAdmins()
    {
        $admins = Admins::where([
            ['roleid', '1'],
            ['disabled', '0'],
        ])->get();
        return $this->prepareArray($admins, 'username');
    }

    /**
     * Get Email Template ID
     *
     * @param string $name
     * @return integer
     */
    protected function getEmailTemplateID($name)
    {
        $emailTemplate = EmailTemplates::where('name', $name)->first();
        if (!is_null($emailTemplate))
        {
            return $emailTemplate->id;
        }
    }

    /**
     * Get available graph options
     *
     * @return array
     */
    private function getGraphsOptions()
    {
        $config = $this->getConfigFile();
        $graphs = [];

        foreach ($config->graphs->available as $value)
        {
            $graphs[$value] = Lang::T($value);
        }
        return $graphs;
    }

    /**
     * Get Configuration file
     *
     * @return JSON object
     */
    private function getConfigFile()
    {
        $file = new FileReader(FileDir::getConfigDir() . DS . 'configuration.json');
        return $file->read();
    }

}
