<?php

namespace MGModule\EasyDCIM\Repositories\Cron;

use Exception;
use MGModule\EasyDCIM\Helpers\CustomFields;
use MGModule\EasyDCIM\Helpers\Emails;
use MGModule\EasyDCIM\Helpers\Logs;
use MGModule\EasyDCIM\MgLibs\Database\Models\Hostings;
use MGModule\EasyDCIM\MgLibs\Database\Models\Product;
use MGModule\EasyDCIM\MgLibs\Database\Models\Servers;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Interfaces\IClient;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;

/**
 * Description of Install
 *
 * @author Mateusz Pawłowski
 */
class Cron extends AbstractRepository
{

    /**
     * Run to check server to synchronize
     */
    public function run()
    {
        $this->checkServices();
    }

    /**
     * Get all Easy DCIM Colocation Products 
     * @return arrray of object insteandof Product
     */
    public function getAllEasyProduct()
    {
        return Product::where('servertype', 'EasyDCIM')->get();
    }

    /**
     * Get all Hostings with to Easy DCIM Colocation Module
     * @return array of object insteandof Product
     */
    public function getHostings()
    {
        $allHostings = [];
        foreach ($this->getAllEasyProduct() as $product)
        {
            $hostings = Hostings::where([
                        ['packageid', $product->id],
                        ['domainstatus', 'Active']
                    ])->get();
            foreach ($hostings as $hosting)
            {
                $allHostings[] = $hosting;
            }
        }
        return $allHostings;
    }

    /**
     * Check and Synchroznize all Active Easy DCIM Colocation 
     */
    public function checkServices()
    {
        foreach ($this->getHostings() as $hosting)
        {
            try
            {
                $this->checkFields($hosting);
            }
            catch (Exception $ex)
            {
                Logs::save('CRON', $hosting, $ex->getMessage(), $hosting->packageid);
            }
        }
    }

    /**
     * Check custom fields and run synchronize
     * 
     * @param object $hosting insteandof Hostings
     * @throw exception
     */
    public function checkFields(Hostings $hosting)
    {
        $orderID  = CustomFields::get($hosting->packageid, $hosting->id, 'OrderID');
        $serverID = CustomFields::get($hosting->packageid, $hosting->id, 'ServerID');
        if (empty($orderID))
        {
            throw new Exception('Order ID is empty!');
        }

        if (empty($serverID))
        {
            $this->synchronizeServices($orderID, $hosting);
        }
    }

    /**
     * Check custom fields and run synchronize
     * 
     * @param integer $orderID object $hosting insteandof Hostings
     */
    private function synchronizeServices($orderID, Hostings $hosting)
    {

        $server                           = $this->getServer($hosting->server);
        $order['customfields']['OrderID'] = $orderID;
        $params['params']                 = array_merge($server, $order);
        $this->checkOrder($params, $hosting);
    }

    /**
     * Check Easy DCIM Order status
     * 
     * @param array $params object $hosting insteandof Hostings
     * @throw exception
     */
    private function checkOrder($params, Hostings $hosting)
    {
        $client = new ClientAdapter($params);
        $api    = new EasyDCIM($client);
        $info   = $api->order->getOrder();

        if (!empty($info->service->related_id))
        {
            CustomFields::set($hosting->packageid, $hosting->id, 'ServerID', $info->service->related_id);
            $device = $this->getDeviceInformation($client, $info->service->related_id);
            $this->saveDedicatedIP($device, $hosting->id);
            Logs::save('CRON', $hosting, 'Synchronize Complete: Server ID #' . $info->service->related_id, $hosting->packageid);
            Emails::sendServerCreateEmail($hosting->packageid, $hosting->id, $device->metadata);
        }
        else
        {
            throw new Exception('Device is not assigned.');
        }
    }

    /**
     * Get and prepare server information
     * 
     * @param integer $serverID
     * @throw exception
     */
    private function getServer($serverID)
    {
        $server = Servers::where('id', $serverID)->first();
        if (!is_null($server))
        {
            return [
                'serverip'       => $server->ipaddress,
                'serverhostname' => $server->hostname,
                'serverpassword' => \decrypt($server->password),
                'secure'         => $server->secure
            ];
        }
        throw new Exception('Server not exists! Server ID #' . $serverID);
    }

    /*
     * Get Easy DCIm device inforamtion
     * 
     * @param string $deviceID
     * @return JSON object
     * @throw Exception API Error
     */

    private function getDeviceInformation(IClient $client, $deviceID)
    {
        $api = new EasyDCIM($client);
        return $api->device->getInformation($deviceID);
    }

    /*
     * Save DedicatedID from Easy dCIM
     * 
     * @param JSON object $device 
     * @throw Exception API Error
     */

    private function saveDedicatedIP($device, $hostingID)
    {
        $hosting = Hostings::where('id', $hostingID)->update([
            'dedicatedip' => $device->metadata->{'IP Address'},
            'assignedips' => $device->metadata->{'Additional IP Addresses'}
        ]);
    }

}
