<?php

namespace MGModule\EasyDCIM\Repositories\Migrations;

use Exception;
use MGModule\EasyDCIM\Helpers\ProductConfiguration;
use MGModule\EasyDCIM\MgLibs\Database\Models\MGOldConfiguration;
use MGModule\EasyDCIM\MgLibs\Database\Models\Product;
use MGModule\EasyDCIM\MgLibs\Database\Models\ProductConfigOptions;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Configuration\Traits\TraitFieldsOption;
use WHMCS\Database\Capsule;

/**
 * Description of Install
 *
 * @author Mateusz PawĹ‚owski
 */
class Migrations extends AbstractRepository
{

    use TraitFieldsOption;

    private $matchFields   = [
        'model'               => 'model',
        'location'            => 'location',
        'template'            => 'osTemplate',
        'baseFeatures'        => 'baseFeatures',
        'monthlyTrafficLimit' => 'bandwidth',
        'additionalIpsNumber' => 'additionalIPAddress',
        'hddSize'             => 'HDDSize',
        'ssdSize'             => 'SSDSize',
        'ramSize'             => 'RAMSize',
        'cpuCores'            => 'CPUCores',
        'hddModel'            => 'HDDModel',
        'ssdModel'            => 'SSDModel',
        'ramModel'            => 'RAMModel',
        'cpuModel'            => 'CPUModel',
        'caBaseFeatures'      => 'caBaseFeatures',
        'caSelectedGraphs'    => 'caSelectedGraphs',
        'caServerActions'     => 'caServerActions',
        'modules'             => 'modules',
        'suspendActions'      => 'suspendActions',
        'unsuspendActions'    => 'unsuspendActions',
        'terminateActions'    => 'terminateActions',
        'serviceAccessLevel'  => 'serviceAccessLevel',
        'templateSuspend'     => 'templateSuspend',
        'templateUnsuspend'   => 'templateUnsuspend',
        'templateTerminate'   => 'templateTerminate',
        'caSidebarActions'    => 'caSidebarActions',
        'templateCreate'      => 'notificationServerCreate',
        'osTemplates'         => 'caOsTemplate',
        'notificationAdminId' => 'notificationAdminId',
        'percentileSoftLimit' => 'percentileSoftLimit',
        'percentileCost'      => 'percentileCost'
    ];
    private $diffrentValue = [
        'baseFeatures',
        'caBaseFeatures',
        'caSelectedGraphs',
        'caServerActions',
        'caSidebarActions',
        'modules',
        'suspendActions',
        'unsuspendActions',
        'terminateActions',
        'serviceAccessLevel',
        'template',
        'osTemplates',
    ];

    /**
     * Get email tempaltes file
     * Add custom template
     */
    public function check()
    {
        $this->updateConfigurableOptions();
        try
        {

            $this->checkExsistsConfig();
            $this->checkAndUpdateTable();
            $this->runMigration();
            $this->checkAndRemovetable();
        }
        catch (Exception $ex)
        {
            return;
        }
        return true;
    }

    /*
     * Check Old Configuration exsist
     *
     * @return void
     * @throw Exception
     */

    private function checkExsistsConfig()
    {
        $checkTable = Capsule::schema()->hasTable('mg_product_configuration');

        if ($checkTable === false)
        {
            throw new \Exception();
        }
    }

    private function checkAndRemovetable()
    {
        Capsule::schema()->dropIfExists('users');
    }

    /*
     * Check is table empty
     */

    private function checkAndUpdateTable()
    {
        if (Capsule::schema()->hasTable('mg_product_configuration')) {
            if (!Capsule::Schema()->hasColumn('mg_product_configuration', 'id')) {
                Capsule::Schema()->table('mg_product_configuration', function ($table) {
                    $table->increments('id');
                });
            }
        }
    }

    /*
     * Get all Easy DCIM Products
     *
     */

    private function getAllEasyProductsID()
    {
        $products    = Product::where('servertype', 'EasyDCIM')->select('id')->get();
        $allProducts = [];
        foreach ($products as $prooduct)
        {
            $allProducts[] = $prooduct->id;
        }
        return $allProducts;
    }

    /*
     * Get Old Configuration
     *
     * @return object
     * @throw Exception
     *
     */

    private function getOldConfiguration($whereIN)
    {
        if (empty($whereIN))
        {
            throw new Exception();
        }
        $config = MGOldConfiguration::whereIn('product_id', $whereIN)->get();
        if ($config->count() == 0)
        {
            throw new Exception();
        }
        return $config;
    }

    private function runMigration()
    {
        $products = $this->getAllEasyProductsID();
        $config   = $this->getOldConfiguration($products);
        foreach ($config as $filed => $value)
        {
            $fieldsValue = json_decode($value->value);
            $fieldName   = $this->matchFields[$value->setting];
            if ($fieldName)
            {

                if (in_array($value->setting, $this->diffrentValue))
                {
                    $methodName = 'set' . ucfirst($value->setting);
                    if (method_exists($this, $methodName))
                    {
$this->{$methodName}($value->product_id, $fieldsValue);
                        continue;
                    }
                }
                ProductConfiguration::set($value->product_id, [$fieldName => $fieldsValue]);
            }
        }

        $this->cleanConfig($config);
    }

    private function updateConfigurableOptions()
    {
        $templatesGroups = $this->getAllOSTemplateConfigs();
        foreach ($templatesGroups as $group)
        {
            foreach ($group->suboptions as $option)
            {
                $newOption = explode("|", $option->optionname);
                if ($newOption[0] != $newOption[1])
                {
                    $option->optionname = $newOption[1] . '|' . $newOption[1];
                    $option->save();
                }
                continue;
            }
        }
    }

    private function getAllOSTemplateConfigs()
    {
        $products   = $this->getAllEasyProductsID();
        $osTemplate = ProductConfigOptions::where('optionname', 'template|Os Template')->with('suboptions', 'group')->whereHas('group', function($q) use($products)
                {
                    $q->whereIn('pid', $products);
                })->get();
        return $osTemplate;
    }

    private function cleanConfig($config)
    {
        foreach ($config as $conf)
        {
            MGOldConfiguration::where([
                ['product_id', $conf->product_id],
                ['setting', $conf->setting],
            ])->delete();
        }
    }

    public function setBaseFeatures($productID, $values)
    {
        $settings = [];
        if (in_array('auto_accept', $values))
        {
            $settings[] = "autoAccept";
        }
        if (in_array('require_pdu', $values))
        {
            $settings[] = "requirePdu";
        }
        if (in_array('require_switch', $values))
        {
            $settings[] = "requireSwitch";
        }
        if (in_array('debug', $values))
        {
            $settings[] = "debug";
        }
        ProductConfiguration::set($productID, ['baseFeatures' => $settings]);
    }

    public function setCaBaseFeatures($productID, $values)
    {
        $settings = [];
        if (in_array('ca_image', $values))
        {
            $settings[] = "caImage";
        }
        if (in_array('ca_model', $values))
        {
            $settings[] = "caModel";
        }
        if (in_array('ca_label', $values))
        {
            $settings[] = "caLabel";
        }
        if (in_array('ca_status', $values))
        {
            $settings[] = "caStatus";
        }
        if (in_array('ca_serialnumber', $values))
        {
            $settings[] = "caSerialnumber";
        }
        if (in_array('ca_purchase_date', $values))
        {
            $settings[] = "caPurchaseDate";
        }
        if (in_array('ca_warranty_months', $values))
        {
            $settings[] = "caWarrantyMonths";
        }
        if (in_array('ca_hostname', $values))
        {
            $settings[] = "caHostname";
        }
        if (in_array('ca_changeHostname', $values))
        {
            $settings[] = "caChangeHostname";
        }
        if (in_array('ca_ips', $values))
        {
            $settings[] = "caIps";
        }

        if (in_array('ca_location', $values))
        {
            $settings[] = "caLocation";
        }

        if (in_array('ca_rack', $values))
        {
            $settings[] = "caRack";
        }
        if (in_array('ca_graphs', $values))
        {
            $settings[] = "caGraphs";
        }
        ProductConfiguration::set($productID, ['caBaseFeatures' => $settings]);
    }

    public function setCaSidebarActions($productID, $values)
    {
        $settings = [];
        if (in_array('change_password', $values))
        {
            $settings[] = "changePassword";
        }
        if (in_array('server_information', $values))
        {
            $settings[] = "serverInformation";
        }

        ProductConfiguration::set($productID, ['caSidebarActions' => $settings]);
    }

    public function setModules($productID, $values)
    {
        $settings = [];
        if (in_array('usagecollector', $values))
        {
            $settings[] = "usageCollector";
        }
        if (in_array('osinstallation', $values))
        {
            $settings[] = "osInstallation";
        }
        if (in_array('dns-manager', $values))
        {
            $settings[] = "dnsManager";
        }

        ProductConfiguration::set($productID, ['modules' => $settings]);
    }

    public function setSuspendActions($productID, $values)
    {
        $settings = [];
        if (in_array('send_an_e_mail_to_the_administrator', $values))
        {
            $settings[] = true;
        }


        ProductConfiguration::set($productID, ['suspendActions' => $settings]);
    }

    public function setUnsuspendActions($productID, $values)
    {
        $settings = [];
        if (in_array('send_an_e_mail_to_the_administrator', $values))
        {
            $settings[] = true;
        }


        ProductConfiguration::set($productID, ['unsuspendActions' => $settings]);
    }

    public function setTerminateActions($productID, $values)
    {
        $settings = [];
        if (in_array('send_an_e_mail_to_the_administrator', $values))
        {
            $settings[] = true;
        }


        ProductConfiguration::set($productID, ['terminateActions' => $settings]);
    }

    public function setCaSelectedGraphs($productID, $values)
    {
        $settings = [];
        if (in_array('Load', $values))
        {
            $settings[] = "ProcessorsLoad";
        }
        if (in_array('Ping', $values))
        {
            $settings[] = "Ping";
        }
        if (in_array('Traffic', $values))
        {
            $settings[] = "AggregateTraffic";
        }
        if (in_array('Status', $values))
        {
            $settings[] = "Status";
        }
        if (in_array('PollerDuration', $values))
        {
            $settings[] = "PollerDuration";
        }

        ProductConfiguration::set($productID, ['caSelectedGraphs' => $settings]);
    }

    public function setCaServerActions($productID, $values)
    {
        $settings = [];
        if (in_array('ca_reboot', $values))
        {
            $settings[] = "caReboot";
        }
        if (in_array('ca_boot', $values))
        {
            $settings[] = "caBoot";
        }
        if (in_array('ca_shutdown', $values))
        {
            $settings[] = "caShutdown";
        }
        if (in_array('ca_sso', $values))
        {
            $settings[] = "caSso";
        }
        if (in_array('ca_kvm_console', $values))
        {
            $settings[] = "caKvmConsole";
        }

        ProductConfiguration::set($productID, ['caServerActions' => $settings]);
    }

    public function setServiceAccessLevel($productID, $values)
    {
        ProductConfiguration::set($productID, ['serviceAccessLevel' => $values[0]]);
    }

    public function setOsTemplates($productID, $values)
    {



        try
        {
            $locationID = $this->getLoationID($productID);

            $tempaltes = $this->getAvailableOSTemplateList($locationID);
            $names     = [];
            foreach ($tempaltes as $template)
            {
                if (in_array($template->id, $values))
                {
                    $names[] = $template->name;
                }
            }

            ProductConfiguration::set($productID, ['caOsTemplate' => $names]);
        }
        catch (Exception $ex)
        {
            //nothing to
        }
    }

    public function setTemplate($productID, $values)
    {
        try
        {
            $locationID = $this->getLoationID($productID);

            $tempaltes = $this->getAvailableOSTemplateList($locationID);
            foreach ($tempaltes as $template)
            {
                if ($template->id == $values)
                {
                    ProductConfiguration::set($productID, ['osTemplate' => $template->name]);
                    break;
                }
            }
        }
        catch (Exception $ex)
        {
            //nothing to
        }
    }

    private function getLoationID($productID)
    {
        $location = MGOldConfiguration::where([['product_id', $productID], ['setting', 'location']])->first();

        return json_decode($location->value);
    }

} 