<?php

namespace MGModule\EasyDCIM\Repositories\Modules;

use Exception;
use MGModule\EasyDCIM\Helpers\Serializer;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Exceptions\FatalError;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\DNS\AddRecord;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\DNS\DataContainer;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Modules\Traits\TraitModules;

/**
 * Description of DnsManager
 *
 * @author Mateusz Pawłowski
 */
class DnsManager extends AbstractRepository
{

    use TraitModules;

    /*
     * Check DNS Manager is active
     * 
     */

    public function check()
    {
        if (!$this->isModuleActive('dns-manager'))
        {
            throw new FatalError('dnsManagerNotActive');
        }
        if (!$this->checkIsPageEnable('dnsManager'))
        {
            throw new FatalError('noAccessDNSManager');
        }
        $this->checkServerID();
    }

    private function checkServerID()
    {
        $serverID = \MGModule\EasyDCIM\Helpers\CustomFields::get($this->params['params']['packageid'], $this->params['params']['serviceid'], 'ServerID');
        if (empty($serverID))
        {
            throw new FatalError('ServerIDIsEmpty');
        }
    }

    /**
     * Get Device information from Easy DCIM
     *
     * @return JSON object
     */
    private function getDeviceInformation()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->device->getInformation();
    }

    public function getIPAddress()
    {
        $device = $this->getDeviceInformation();
        if (isset($device->ip_addresses))
        {
            $ipAddresses = $device->ip_addresses;
        }
        else
        {
            $additional  = explode(',', $device->metadata->{'Additional IP Addresses'});
            $primary     = [$device->metadata->{'IP Address'}];
            $ipAddresses = array_merge($primary, $additional);
        }
        return $ipAddresses;
    }

    public function getDNS()
    {
        $data    = [];
        $records = $this->getAllrDNSRecords();
        foreach ($records->rdns as $key => $value)
        {
            $data[] = $this->dataTablesParseRow('dnsrow', $value);
        }

        return ['data' => $data];
    }

    public function getAllrDNSRecords()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->dnsManager->getRecords();
    }

    public function saveDNS()
    {
        $field         = Serializer::unserialize($this->params['formContainer']);
        $rDNSModel     = new AddRecord();
        $rDNSModel->setIp($field['rdns']['ip'])
                ->setRdata($field['rdns']['rdata'])
                ->setTtl($field['rdns']['ttl']);
        $dataContainer = new DataContainer();
        $dataContainer->setData($rDNSModel);


        $api      = new EasyDCIM(new ClientAdapter($this->params));
        $response = $api->dnsManager->createRecord($dataContainer);
        if ($response->message != "Success")
        {
            throw new Exception(($response->message) ?: Lang::T('recodCannotAdd'));
        }
        return sprintf(Lang::T('recordAdded'), $field['rdns']['rdata'] . " (" . $field['rdns']['ip'] . ")");
    }

    public function deleteDNS()
    {
        $field = Serializer::unserialize($this->params['formContainer']);
        $api   = new EasyDCIM(new ClientAdapter($this->params));
        $api->dnsManager->deleteRecord($field['target']);
        return Lang::T('recordDeleted');
    }

    public function updateDNS()
    {
        $field         = Serializer::unserialize($this->params['formContainer']);
        $rDNSModel     = new AddRecord();
        $rDNSModel->setRdata($field['rdns']['rdata']);
        $dataContainer = new DataContainer();
        $dataContainer->setData($rDNSModel);

        $api      = new EasyDCIM(new ClientAdapter($this->params));
        $response = $api->dnsManager->updateRecord($field['target'], $dataContainer);
        if ($response->message != "Success")
        {
            throw new Exception(($response->message) ?: Lang::T('recodCannotAdd'));
        }
        return Lang::T('recordEdited');
    }

}
