<?php

namespace MGModule\EasyDCIM\Repositories\Modules;

use Exception;
use MGModule\EasyDCIM\Helpers\OsTemplate;
use MGModule\EasyDCIM\Helpers\ProductConfiguration;
use MGModule\EasyDCIM\Helpers\SystemLogo;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Exceptions\FatalError;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Os\Configuration;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Models\Os\InstallParams;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Modules\Traits\TraitModules;

/**
 * Description of DnsManager
 *
 * @author Mateusz Pawłowski
 */
class OsInstallation extends AbstractRepository
{

    use TraitModules;

    /*
     * Check Os Installation Module is active
     * 
     * Return simple inforamtion
     * 
     * @return array
     */

    public function check()
    {
        if (!$this->isModuleActive('os'))
        {
            throw new FatalError('osInstallationNotActive');
        }
        if (!$this->checkIsPageEnable('osInstallation'))
        {
            throw new FatalError('noAccessOsInstallation');
        }
        return $this->getInstallInformation();
    }

    private function getInstallInformation()
    {
        $vars['hostname'] = $this->params['params']['domain'];
        $vars['device']   = $this->getDeviceInformation();
        $vars['os']       = $this->getAvailableOS($osInformation);
        return $vars;
    }

    private function getDeviceInformation()
    {
        $api    = new EasyDCIM(new ClientAdapter($this->params));
        $device = $api->device->getInformation();
        return $device->metadata;
    }

    public function getStatus()
    {
        $osInformation = $this->getInformationFromAPI();
        return $this->getSimpleinformation($osInformation)['duringInstallation'];
    }

    /*
     * Get OS Inforamtion from Easy DCIM
     * 
     * @return JSON object
     * @throw Exception
     */

    private function getInformationFromAPI()
    {
        try
        {
            $api = new EasyDCIM(new ClientAdapter($this->params));
            return $api->os->installInformation();
        }
        catch (Exception $ex)
        {
            if ($ex->getMessage() == "Unable to find appropriate server. ")
            {
                throw new Exception(Lang::absoluteT('apiError', 'noAvailableTemplates'));
            }

            throw $ex;
        }
    }

    private function getSimpleinformation($osInformation)
    {
        $fields = [];


        foreach ($osInformation->device->metadata as $metadata)
        {
            switch ($metadata->field->slug)
            {
                case 'hostname':
                    $fields['hostname']   = $metadata->value;
                    break;
                case 'mac.address':
                    $fields['macAddress'] = $metadata->value;
                    break;
                case 'os':
                    $fields['os']         = $metadata->value;
                    break;
            }
        }
        $fields['duringInstallation'] = $osInformation->moduleInfo->duringInstallation;
        return $fields;
    }

    private function getAvailableOS($osInformation)
    {
        $config = $this->getProductConfig();

        $osList = [];
        foreach ($this->getOSList() as $key => $value)
        {
            if (in_array($value->name, $config['caOsTemplate']))
            {
                $osList[$value->name] = [
                    'id'   => $value->name,
                    'name' => $value->name,
                    'logo' => SystemLogo::getLogo($value->name)
                ];
            }
        }
        return $osList
        ;
    }

    private function getProductConfig()
    {
        return ProductConfiguration::get($this->params['params']['packageid']);
    }

    private function getOSList()
    {

        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->os->getTemplateList();
    }

    public function rebuild()
    {
        $this->checkISNotEmpty();
        $info = $this->installOS();
        if ($info == true)
        {
            return ['success' => Lang::T('installStarted')];
        }
        return ['error' => $info];
    }

    private function checkISNotEmpty()
    {
        if (empty($this->params['osTemplate']))
        {
            throw new Exception(Lang::T('selectOSTempalte'));
        }
    }

    private function installOS()
    {
        $install    = new InstallParams();
        $templateID = (new OsTemplate($this->params))->getOSTemplateID($this->params['osTemplate']);

        $install->setTemplate($templateID)
                ->setHostname($this->params['mgHostname'])
                ->setUsername($this->params['mgUsername'])
                ->setPassword($this->params['mgPassword'])
                ->setRootPassword($this->params['mgRootPassword']);

        $configurtation = new Configuration();
        $configurtation->setConfiguration($install);

        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->os->install($configurtation);
    }

}
