<?php

namespace MGModule\EasyDCIM\Repositories\Product\Fields;

use Exception;
use MGModule\AdvancedBilling\classes\Account;
use MGModule\AdvancedBilling\classes\Product;
use MGModule\EasyDCIM\Helpers\AdvancedBilling as ABHelper;
use MGModule\EasyDCIM\Helpers\FileDir;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use const DS;

/**
 * Description of AdvancedBilling
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class AdvancedBilling extends AbstractRepository
{
    /*
     * Advanced Billing main path
     * 
     * @var string $abPath
     */

    private $abPath;

    /*
     * Send params to parent and set Advanced Billing module path
     * 
     * @var string $abPath
     */

    public function __construct($params)
    {
        $this->abPath = FileDir::getModulesDir() . DS . 'addons' . DS . 'AdvancedBilling';
        parent::__construct($params);
    }

    /*
     * Get Field with template
     * 
     * @return array
     */

    public function getField()
    {
        $template = $this->getTemplate();
        //check AB
        if ($template)
        {
            return [
                Lang::T('AdvancedBilling') => $template
            ];
        }
        return [];
    }

    /*
     * Check Advanced Billing is active and get Advanced Billing widget
     *      
     * @return string 
     */

    private function getTemplate()
    {
        try
        {
            if (ABHelper::isActive())
            {
                return $this->prepareTemplate();
            }
        }
        catch (Exception $ex)
        {
            return;
        }
    }

    /*
     * Get records and preapare advanced billing template string 
     * 
     * @return string 
     */

    private function prepareTemplate()
    {
        $record   = $this->getRecord();
        $currency = getCurrency($this->params['params']['userid'])['suffix'];
        $vars     = [
            'bandwidth'       => $record['bandwidth']['usage'] . ' ' . $record['units']['bandwidth'],
            'bandwidthPrice'  => number_format($record['bandwidth']['price'], 2, ".", "") . $currency,
            'percentile'      => $record['percentile']['usage'] . ' ' . $record['units']['percentile'],
            'percentilePrice' => number_format($record['percentile']['price'], 2, ".", "") . $currency,
            'powerUsage'      => $record['powerUsage']['usage'] . ' ' . $record['units']['powerUsage'],
            'powerUsagePrice' => number_format($record['powerUsage']['price'], 2, ".", "") . $currency,
            'total'           => number_format($record['bandwidth']['price'] + $record['percentile']['price'] + $record['powerUsage']['price'], 2, ".", "") . $currency,
        ];
        return $this->parseTemplateToString('advancedBilling', '/pages/advancedBilling', $vars);
    }

    /*
     * Prepare a information about usage
     * 
     * @return array
     */

    private function getRecord()
    {
        $resource = $this->getResource();
        $total    = $this->getTotal();
        return array_merge($total, $resource);
    }

    /*
     * Get information about product from Advanced Billing
     * 
     * @thorw silence error  
     */

    private function getResource()
    {
        if (!file_exists($this->abPath . DS . 'classes' . DS . 'Product.php'))
        {
            throw new Exception();
        }
        require_once $this->abPath . DS . 'classes' . DS . 'Product.php';
        $product            = new Product($this->params['params']['packageid']);
        $productInformation = $product->getResourceSettings();
        return $this->getUnits($productInformation);
    }

    /*
     * Get information about usage from Advanced Billing
     * 
     * @thorw silence error  
     */

    private function getTotal()
    {
        if (!file_exists($this->abPath . DS . 'classes' . DS . 'Account.php'))
        {
            throw new Exception();
        }
        require_once $this->abPath . DS . 'classes' . DS . 'Account.php';
        $account = new Account($this->params['params']['serviceid']);

        $item = $account->getSummary();
        if (empty($item['total']))
        {
            throw new Exception();
        }

        return $item['total'];
    }

    /*
     * Get information about units from Resource information
     * 
     * @return array
     */

    private function getUnits($products)
    {
        $units = [];
        foreach ($products as $product)
        {
            $units['units'][$product['name']] = $product['unit'];
        }
        return $units;
    }

}
