<?php

namespace MGModule\EasyDCIM\Repositories\Product\Fields;

use Exception;
use Carbon\Carbon;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\Adapters\ClientAdapter;
use MGModule\EasyDCIM\MgLibs\EasyDCIM\EasyDCIM;
use MGModule\EasyDCIM\MgLibs\Lang;
use MGModule\EasyDCIM\MgLibs\Models\AbstractRepository;
use MGModule\EasyDCIM\Repositories\Graphs\Graph;

/**
 * Description of AdvancedBilling
 *
 * @author Mateusz Pawłowski <mateusz.pa@modulesgarden.com>
 */
class DeviceInformation extends AbstractRepository
{
    /*
     * Get Field with template
     * 
     * @return array
     */

    public function getField()
    {
        $template = $this->getTemplate();

        if ($template)
        {
            return [
                Lang::T('DeviceInformation') => $template
            ];
        }
        return [];
    }

    /*
     * Check Server ID is not empty and get template
     * 
     * @return string 
     */

    private function getTemplate()
    {
        try
        {
            $this->checkServer();
            return $this->prepareTemplate();
        }
        catch (Exception $ex)
        {
            return;
        }
    }

    /*
     * Check is server ID field is not empty
     * 
     * @throw silence error
     */

    private function checkServer()
    {
        if (empty($this->params['params']['customfields']['ServerID']))
        {
            throw new Exception();
        }
    }

    /*
     * Get records and preapare graphs template string 
     * 
     * @return string 
     */

    private function prepareTemplate()
    {

        $vars                = $this->getDateInformation();
        $vars['mainJSONURL'] = $this->getPageURL();
        return $this->parseTemplateToString('deviceInformation', '/pages/deviceInformation', $vars);
    }

    /*
     * Get data range for account
     * 
     * @return arrays 
     */

    private function getDateInformation()
    {
        $device = $this->getDeviceInformation();
        $date   = $this->getDateToGraph();

        return [
            'startDateBandwidth'  => $date['startDateBandwidth'],
            'endDateBandwidth'    => $date['endDateBandwidth'],
            'startMaxDate'        => $date['startMaxDate'],
            'endMaxDate'          => $date['endMaxDate'],
            'endDisplayDate'      => $date['endDisplayDate'],
            'startDisplayDate'    => $date['startDisplayDate'],
            'deviceStatus'        => ucfirst($device->device_status),
            'model'               => $device->model,
            'label'               => $device->label,
            'serial'              => $device->serialnumber1,
            'status'              => $device->status,
            'purchaseDate'        => $device->purchase_date,
            'warrantyMonths'      => $device->warranty_months,
            'locationName'        => $device->location->name,
            'locationAddress'     => $device->location->address,
            'locationPhone'       => $device->location->phone,
            'locationDescription' => $device->location->description,
            'ipAddresses'         => (isset($device->ip_addresses)) ?  $device->ip_addresses : [$device->metadata->{'IP Address'}]
        ];
    }

    private function getDateToGraph()
    {
        $graph = new Graph($this->params);
        $displayDate = $graph->getDisplayDate();
        $bandwidthDate = $graph->getDataRange();
        $rangeDate = $graph->getDateMax();

        return [
            'startDisplayDate' => $displayDate['startDisplayDate'],
            'endDisplayDate' => $displayDate['endDisplayDate'],
            'startMaxDate' => $rangeDate['startMaxDate'],
            'endMaxDate' => $rangeDate['endMaxDate'],
            'startDateBandwidth' => $bandwidthDate['startDate'],
            'endDateBandwidth' => $bandwidthDate['endDate']

        ];
    }

    /**
     * Get Device information from Easy DCIM
     *
     * @return JSON object
     */
    private function getDeviceInformation()
    {
        $api = new EasyDCIM(new ClientAdapter($this->params));
        return $api->device->getInformation();
    }

    /*
     * Get JSON request URL
     * Required to get graphs
     * 
     * @return string 
     */

    private function getPageURL()
    {
        $params = [
            'userid'        => $this->params['params']['userid'],
            'productselect' => $this->params['params']['serviceid'],
            'id'            => $this->params['params']['serviceid'],
            'mg-page'       => 'Product',
            'json'          => 'true'
        ];
        return 'clientsservices.php?' . http_build_query($params);
    }

}
